package cn.iocoder.yudao.module.shipment.listener;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentSummeryNewExcelExportPushEvent;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.CurrencyRespDTO;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableBoxReportPageVO;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableBoxReportQueryVO;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableBoxReportVO;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableCurrencyAmount;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;

@Component("ShipmentSummeryNewExcelExportListener")
@AllArgsConstructor
@Slf4j
public class ShipmentSummeryNewExcelExportListener {

    private ReceivableService receivableService;

    private UeProperties ueProperties;

    private FileService fileService;

    private CurrencyApi currencyApi;

    @EventListener(ShipmentSummeryNewExcelExportPushEvent.class)
    public void shipmentSummeryNewExcelExportPushEvent(ShipmentSummeryNewExcelExportPushEvent event) {
        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                PageVO page = new PageVO();
                page.setRows(10000);
                page.setPage(1);

                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/shipmentSummery/excel/new/");
                File fileDir = new File(dir);
                if (!fileDir.exists()) {
                    fileDir.mkdirs();
                }
                String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("shipment_summery.xlsx");
                JSONObject jsonObject = JSONObject.parseObject(event.getRequestParams());
                String queryString = jsonObject.getString("query");
                ReceivableBoxReportQueryVO query = JSONObject.parseObject(queryString, ReceivableBoxReportQueryVO.class);
                ReceivableBoxReportVO boxPage = receivableService.getReceivableBoxReport(query, page);
                List<ReceivableBoxReportPageVO> records = boxPage.getList();
                Map headMap = new HashMap();
                if (CollectionUtil.isNotEmpty(records)) {
                    InputStream inputStream = getClass().getClassLoader().getResourceAsStream(ueProperties.getTemplatesUrl() + "/box_finance_new.xlsx");
                    String nameZh = "新自编号报表汇总统计";
                    String nameEn = "New Summary Statistics of Shipment";
                    String sheetName = I18nMessage.getLang() == 0 ? nameZh : nameEn;

                    List<Map<String, Object>> list = new ArrayList<>();

                    for (ReceivableBoxReportPageVO item : records) {
                        String jsTime = DateUtils.formatDate(item.getSlSettledTime());

                        Map<String, Object> map = new HashMap<>();
                        map.put("jsTime", jsTime);
                        map.put("selfNo", item.getSelfNo());
                        map.put("huizong", item.getReceivableTotalSummary());
                        map.put("jsBee", item.getReceivableTotalSettle());
                        map.put("yfFee", item.getReceivableDetailFreight());
                        map.put("qgfFee", item.getReceivableDetailClearance());
                        map.put("ewFee", item.getReceivableDetailOtherFee());
                        map.put("myfFee", item.getReceivableDestCountryFreight());
                        map.put("mqgfFee", item.getReceivableDestCountryClearance());
                        map.put("mewFee", item.getReceivableDestCountryOtherFee());
                        map.put("mewfFee", item.getReceivableDestCountryOtherFeeSub());
                        map.put("syfFee", item.getReceivableStartCountryFreight());
                        map.put("sqgfFee", item.getReceivableStartCountryClearance());
                        map.put("sewFee", item.getReceivableStartCountryOtherFee());
                        map.put("sewfFee", item.getReceivableStartCountryOtherFeeSub());
                        map.put("zyfFee", item.getReceivableDiscountFreight());
                        map.put("zqgfFee", item.getReceivableDiscountClearance());
                        map.put("zewFee", item.getReceivableDiscountOtherFee());
                        map.put("wyfFee", item.getReceivableUnsettledFreight());
                        map.put("wqgfFee", item.getReceivableUnsettledClearance());
                        map.put("wewFee", item.getReceivableUnsettledOtherFee());

                        map.put("hxbl", item.getWriteOffPercent() + "%");
                        Integer ladingBillStatus = item.getLadingBillStatus();
                        if (ladingBillStatus == null) {
                            map.put("tdzz", "");
                        } else if (ladingBillStatus == 0) {
                            map.put("tdzz", "未完成");
                        } else if (ladingBillStatus == 1) {
                            map.put("tdzz", "部分完成");
                        } else if (ladingBillStatus == 2) {
                            map.put("tdzz", "已完成");
                        }
                        map.put("dgsj", DateUtils.formatDate(item.getDgDate()));
                        map.put("qgsh", DateUtils.formatDate(item.getQgDate()));
                        map.put("xgsj", DateUtils.formatDate(item.getUlWarehouseTime()));
                        list.add(map);
                    }
                    ReceivableBoxReportVO.Amount pageAmount = boxPage.getPageAmount();
                    Map<Integer, CurrencyRespDTO> allCurrency = currencyApi.getAllCurrency();
                    headMap.put("totaHuizong", getHuizong(pageAmount.getReceivableTotalSummary(), allCurrency));
                    headMap.put("totalJsBee", getHuizong(pageAmount.getReceivableTotalSettle(), allCurrency));
                    headMap.put("totalYfFee", getHuizong(pageAmount.getReceivableDetailFreight(), allCurrency));
                    headMap.put("totalQgfFee", getHuizong(pageAmount.getReceivableDetailClearance(), allCurrency));
                    headMap.put("totalEwFee", getHuizong(pageAmount.getReceivableDetailOtherFee(), allCurrency));
                    headMap.put("totalMfyFee", getHuizong(pageAmount.getReceivableDestCountryFreight(), allCurrency));
                    headMap.put("totalMqgfFee", getHuizong(pageAmount.getReceivableDestCountryClearance(), allCurrency));
                    headMap.put("totalMewFee", getHuizong(pageAmount.getReceivableDestCountryOtherFee(), allCurrency));
                    headMap.put("totalMewfFee", getHuizong(pageAmount.getReceivableDestCountryOtherFeeSub(), allCurrency));
                    headMap.put("totalSyfFee", getHuizong(pageAmount.getReceivableStartCountryFreight(), allCurrency));
                    headMap.put("totalSqgfFee", getHuizong(pageAmount.getReceivableStartCountryClearance(), allCurrency));
                    headMap.put("totalSewFee", getHuizong(pageAmount.getReceivableStartCountryOtherFee(), allCurrency));
                    headMap.put("totalSewfFee", getHuizong(pageAmount.getReceivableStartCountryOtherFeeSub(), allCurrency));
                    headMap.put("totalZyfFee", getHuizong(pageAmount.getReceivableDiscountFreight(), allCurrency));
                    headMap.put("totalZqgfFee", getHuizong(pageAmount.getReceivableDiscountClearance(), allCurrency));
                    headMap.put("totalZewFee", getHuizong(pageAmount.getReceivableDiscountOtherFee(), allCurrency));
                    headMap.put("totalWyfFee", getHuizong(pageAmount.getReceivableUnsettledFreight(), allCurrency));
                    headMap.put("totalWqgfFee", getHuizong(pageAmount.getReceivableUnsettledClearance(), allCurrency));
                    headMap.put("totalWewFee", getHuizong(pageAmount.getReceivableUnsettledOtherFee(), allCurrency));

                    ExcelWriter excelWriter = EasyExcel.write(dir + fileName).withTemplate(inputStream).build();
                    WriteSheet sheet = EasyExcel.writerSheet(0).build();
                    sheet.setSheetName(sheetName);
                    FillConfig fillConfig = FillConfig.builder().forceNewRow(true).build();
                    excelWriter.fill(headMap, sheet);
                    excelWriter.fill(list, fillConfig, sheet);
                    excelWriter.finish();
                    inputStream.close();
                } else {
                    ExcelWriter excelWriter = EasyExcel.write(dir + fileName).build();
                    WriteSheet sheet = EasyExcel.writerSheet(0).build();
                    excelWriter.write(null, sheet);
                    excelWriter.finish();
                }
                // 获取到临时文件
                File file = new File(dir + fileName);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());
            } catch (Exception e) {
                // TODO 测试阶段打印堆栈错误信息，便于分析原因
                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }
    }

    private String getHuizong(List<ReceivableCurrencyAmount> receivableTotalSummary,Map<Integer, CurrencyRespDTO> allCurrency) {
        StringBuilder sb = new StringBuilder();
        for (ReceivableCurrencyAmount receivableCurrencyAmount : receivableTotalSummary) {
            sb.append(allCurrency.get(receivableCurrencyAmount.getCurrencyId().intValue()).getFuhao()).append(" ").append(receivableCurrencyAmount.getAmount()).append(", ");
        }
        return sb.substring(0, sb.length() - 2);
    }
}
